#ifndef __SOPHIACRADLE_EXPRESSION_HPP
#define __SOPHIACRADLE_EXPRESSION_HPP

#include <SophiaFramework.hpp>
#include "Array.hpp"
#include "KeySequence.hpp"
#include "Calculator.hpp"

#define MAX_FLOAT_ORDER 30      // ł̃I[_[͖͂

// 萔ނ̒`
#define     MATH_CONSTANT_PI        (3.14159265358979323846)
#define     MATH_CONSTANT_E         (2.71828182845904523536)
#define     MATH_UNIT_TERA          (1.0E+12)
#define     MATH_UNIT_GIGA          (1.0E+9)
#define     MATH_UNIT_MEGA          (1.0E+6)
#define     MATH_UNIT_KILO          (1.0E+3)
#define     MATH_UNIT_MILLI         (1.0E-3)
#define     MATH_UNIT_MICRO         (1.0E-6)
#define     MATH_UNIT_NANO          (1.0E-9)
#define     MATH_UNIT_PICO          (1.0E-12)
#define     MATH_UNIT_FEMTO         (1.0E-15)

// \̂߂̃NX
template <typename T>
class ExpressionElement {
public:
    enum RawExpressionType {
        // ǉ͊eނ̍Ōɒǉ邱(dv)
        RAWTYPE_NONE = 0x0000,          // `
        RAWTYPE_VALUE = 0x0001,         // l
        RAWTYPE_LEFT = 0x0002,          // (
        RAWTYPE_RIGHT = 0x0003,         // )
        RAWTYPE_SEPARATOR = 0x0004,     // ,
        RAWTYPE_FEXP = 0x0005,          // w\ E
        RAWTYPE_OPERATOR = 0x0010,      // ȉ͉Zq
        RAWTYPE_PLUS = 0x0010,          // +
        RAWTYPE_MINUS = 0x0011,         // -
        RAWTYPE_MUL = 0x0012,           // *
        RAWTYPE_DIV = 0x0013,           // /
        RAWTYPE_OPOW = 0x0014,          // ^
        RAWTYPE_FACTORIAL = 0x0015,     // !
        RAWTYPE_MOD = 0x0016,           // mod
        RAWTYPE_AND = 0x0017,           // and
        RAWTYPE_OR = 0x0018,            // or
        RAWTYPE_XOR = 0x0019,           // xor
        RAWTYPE_NOT = 0x001A,           // not
        RAWTYPE_NPR = 0x001B,           // nPr
        RAWTYPE_NCR = 0x001C,           // nCr
        RAWTYPE_SMUL = 0x001D,          // ł * (p)
        RAWTYPE_FUNCTION = 0x0100,      // ȉ͊֐
        RAWTYPE_POW = 0x0100,           // pow
        RAWTYPE_EXP = 0x0101,           // exp
        RAWTYPE_LN = 0x0102,            // ln
        RAWTYPE_LOG = 0x0103,           // log
        RAWTYPE_SQRT = 0x0104,          // sqrt
        RAWTYPE_CBRT = 0x0105,          // cbrt
        RAWTYPE_SIN = 0x0106,           // sin
        RAWTYPE_COS = 0x0107,           // cos
        RAWTYPE_TAN = 0x0108,           // tan
        RAWTYPE_ASIN = 0x0109,          // asin
        RAWTYPE_ACOS = 0x010A,          // acos
        RAWTYPE_ATAN = 0x010B,          // atan
        RAWTYPE_SINH = 0x010C,          // sinh
        RAWTYPE_COSH = 0x010D,          // cosh
        RAWTYPE_TANH = 0x010E,          // tanh
        RAWTYPE_ASINH = 0x010F,         // asinh
        RAWTYPE_ACOSH = 0x0110,         // acosh
        RAWTYPE_ATANH = 0x0111,         // atanh
        RAWTYPE_ABS = 0x0112,           // abs
        RAWTYPE_INT = 0x0113,           // int
        RAWTYPE_FIX = 0x0114,           // fix
        RAWTYPE_GAMMA = 0x0115,         // Gamma
        RAWTYPE_FRAC = 0x0116,          // frac
        RAWTYPE_JN = 0x0117,            // jn
        RAWTYPE_YN = 0x0118             // yn
    };
    enum ExpressionType {
        // ǉ͍Ōɍs(dv)
        TYPE_NONE = 0x0000,             // `
        TYPE_VALUE = 0x0001,            // l
        TYPE_PROCESSED = 0x1000,        // 
        TYPE_PLUS = 0x1000,             // + ()
        TYPE_MINUS = 0x1001,            // - ()
        TYPE_ADD = 0x1002,              // +
        TYPE_SUB = 0x1003,              // -
        TYPE_MUL = 0x1004,              // *
        TYPE_DIV = 0x1005,              // /
        TYPE_MOD = 0x1006,              // mod
        TYPE_AND = 0x1007,              // and
        TYPE_OR = 0x1008,               // or
        TYPE_XOR = 0x1009,              // xor
        TYPE_NOT = 0x100A,              // not
        TYPE_FACTORIAL = 0x100B,        // !
        TYPE_DFACTORIAL = 0x100C,       // !!
        TYPE_NPR = 0x100D,              // nPr
        TYPE_NCR = 0x100E,              // nCr
        TYPE_FUNCTION = 0x100F,         // ȉ͊֐
        TYPE_POW = 0x100F,              // pow
        TYPE_EXP = 0x1010,              // exp
        TYPE_LN = 0x1011,               // ln
        TYPE_LOG = 0x1012,              // log
        TYPE_SQRT = 0x1013,             // sqrt
        TYPE_CBRT = 0x1014,             // cbrt
        TYPE_SIN = 0x1015,              // sin
        TYPE_COS = 0x1016,              // cos
        TYPE_TAN = 0x1017,              // tan
        TYPE_ASIN = 0x1018,             // asin
        TYPE_ACOS = 0x1019,             // acos
        TYPE_ATAN = 0x101A,             // atan
        TYPE_SINH = 0x101B,             // sinh
        TYPE_COSH = 0x101C,             // cosh
        TYPE_TANH = 0x101D,             // tanh
        TYPE_ASINH = 0x101E,            // asinh
        TYPE_ACOSH = 0x101F,            // acosh
        TYPE_ATANH = 0x1020,            // atanh
        TYPE_ABS = 0x1021,              // abs
        TYPE_INT = 0x1022,              // int
        TYPE_FIX = 0x1023,              // fix
        TYPE_GAMMA = 0x1024,            // Gamma
        TYPE_FRAC = 0x1025,             // frac
        TYPE_JN = 0x1026,               // jn
        TYPE_YN = 0x1027                // yn
    };

private:
    UInt16 _type;   // ʂێ
    T _val;         // lێ

public:
    explicit ExpressionElement(Void);
    explicit ExpressionElement(const ExpressionElement<T> &element);
    ~ExpressionElement(Void);
    const ExpressionElement<T> &operator=(const ExpressionElement<T> &element);
    Void SetType(UInt16 type);
    Void SetValue(T val);
    UInt16 GetType(Void) const;
    T GetValue(Void) const;
    static UInt16 InterpretKeyCode(KeyType::KeyCode code);
    static Bool IsRawOperator(UInt16 type);
    static Bool IsRawFunction(UInt16 type);
};

template <typename T>
inline ExpressionElement<T>::ExpressionElement(Void) : _type(TYPE_NONE), _val(0)
{
}

template <typename T>
inline ExpressionElement<T>::ExpressionElement(const ExpressionElement<T> &element) : _type(element._type), _val(element._val)
{
}

template <typename T>
inline ExpressionElement<T>::~ExpressionElement(Void)
{
}

template <typename T>
inline const ExpressionElement<T> &ExpressionElement<T>::operator=(const ExpressionElement<T> &element)
{
    _type = element._type;
    _val = element._val;
    return *this;
}

template <typename T>
inline Void ExpressionElement<T>::SetType(UInt16 type)
{
    _type = type;
    _val = 0;
}

template <typename T>
inline Void ExpressionElement<T>::SetValue(T val)
{
    _type = TYPE_VALUE;
    _val = val;
}

template <typename T>
inline UInt16 ExpressionElement<T>::GetType(Void) const
{
    return _type;
}

template <typename T>
inline T ExpressionElement<T>::GetValue(Void) const
{
    return _val;
}

// L[R[h -> \̕ϊe[u
template <typename T>
inline UInt16 ExpressionElement<T>::InterpretKeyCode(KeyType::KeyCode code)
{
    UInt16 type(RAWTYPE_NONE);
    if ((code & KeyType::KEYCLASS_PARENTHESIS)) {
        type = static_cast<UInt16>(code - KeyType::KEYCLASS_PARENTHESIS + RAWTYPE_LEFT);
    }
    else if (code == KeyType::KEYCODE_FUNCTION_SEPARATOR) {
        type = RAWTYPE_SEPARATOR;
    }
    else if (code & KeyType::KEYCLASS_OPERATOR) {
        type = static_cast<UInt16>(code - KeyType::KEYCLASS_OPERATOR + RAWTYPE_OPERATOR);
    }
    else if (code & KeyType::KEYCLASS_FUNCTION) {
        type = static_cast<UInt16>(code - (KeyType::KEYCLASS_FUNCTION + 1) + RAWTYPE_FUNCTION);
    }
    return type;
}

template <typename T>
inline Bool ExpressionElement<T>::IsRawOperator(UInt16 type)
{
    return (type >= RAWTYPE_OPERATOR && type < RAWTYPE_FUNCTION);
}

template <typename T>
inline Bool ExpressionElement<T>::IsRawFunction(UInt16 type)
{
    return (type >= RAWTYPE_FUNCTION && !(type & TYPE_PROCESSED));
}

// \ێNX
template <typename T>
class Expression : public static_exception<SFCError> {
private:
    Array< ExpressionElement<T> > _expression;
    UInt16 _radixBit;   // ̃rbg
    T _ans;             // 
    Bool _hasUnit;      // Pʂoꂵ

public:
    explicit Expression(UInt16 radixBit = 0);    // radixBit = 0  [hz
    explicit Expression(const Expression<T> &ob) static_throws;
    virtual ~Expression(Void);
    const Expression<T> &operator=(const Expression<T> &ob) static_throws;
    Void SetRadixBit(UInt16 radixBit);
    Void SetAns(T ans);
    Void Set(KeySequenceConstRef keySeq) static_throws; // ͂t|[h\Lɕϊ (OJp)
    SInt32 GetSize(Void) const;
    const ExpressionElement<T> &Get(SInt32 index) const;
    Bool HasUnit(Void) const;
private:
    virtual Void Copy(const Expression<T> &ob) static_throws;
    Void CreateMathExpression(KeySequenceConstRef keySeq) static_throws;
    Void CheckParenthesesCount(Void) const static_throws;
    SInt32 GetStartPosition(SInt32 index) const;
    SInt32 GetEndPosition(SInt32 index) const;
    Void Preprocess(Void) static_throws;
    Void ConvertUnaryOperator(Void) static_throws;
    Void ConvertBinaryOperator(Void) static_throws;
    Void ConvertFunction(Void) static_throws;
};

template <typename T>
inline Expression<T>::Expression(UInt16 radixBit) : _radixBit(radixBit), _ans(0), _hasUnit(false)
{
}

template <typename T>
inline Expression<T>::Expression(const Expression<T> &ob) static_throws
{
    Copy(ob);
}

template <typename T>
inline const Expression<T> &Expression<T>::operator=(const Expression<T> &ob) static_throws
{
    Copy(ob);
    return *this;
}

template <typename T>
inline Expression<T>::~Expression(Void)
{
    _expression.Clear();
}

template <typename T>
inline Void Expression<T>::SetRadixBit(UInt16 radixBit)
{
    _radixBit = radixBit;
}

template <typename T>
inline Void Expression<T>::SetAns(T ans)
{
    _ans = ans;
}

// L[͂t|[h\Lɕϊ (OJp)
template <typename T>
inline Void Expression<T>::Set(KeySequenceConstRef keySeq) static_throws
{
    _hasUnit = false;
    static_throw(SFERR_NO_ERROR);
    _expression.Clear();

    CreateMathExpression(keySeq);
    CheckParenthesesCount();
    Preprocess();
    ConvertUnaryOperator();
    ConvertBinaryOperator();
    ConvertFunction();
}

template <typename T>
inline Void Expression<T>::Copy(const Expression<T> &ob) static_throws
{
    _radixBit = ob._radixBit;
    _ans = ob._ans;
    _hasUnit = ob._hasUnit;
    static_throw(_expression.Set(ob._expression));
}

// L[͂𐔊wI\ɉ߂ȂB
template <typename T>
inline Void Expression<T>::CreateMathExpression(KeySequenceConstRef keySeq) static_throws
{
    KeyType key;
    KeyType::KeyCode code;
    ExpressionElement<T> element;
    UInt64 num(0);
    SInt32 i;
    SInt32Const size(keySeq.GetSize());
    Bool flag(false);

    for (i = 0; i < size; ++i) {
        key = keySeq.GetKeyType(i);
        code = key.GetKeyCode();
        if (code & KeyType::KEYCLASS_NUMBER) {
            // lɕϊ
            if (!flag) {
                flag = true;
                num = static_cast<UInt64>(code - KeyType::KEYCODE_NUMBER_0);
            }
            else {
                num <<= _radixBit;
                num |= static_cast<UInt64>(code - KeyType::KEYCODE_NUMBER_0);
            }
            continue;
        }
        if (flag) {
            flag = false;
            element.SetValue(static_cast<T>(num));
            static_throw(_expression.Append(element));
        }
        if (static_catch() == SFERR_NO_ERROR) {
            if (code == KeyType::KEYCODE_MEMORY_ANS) {
                // 
                element.SetValue(_ans);
            }
            else {
                element.SetType(ExpressionElement<T>::InterpretKeyCode(code));
            }
            static_throw(_expression.Append(element));
        }
        if (static_catch() != SFERR_NO_ERROR) {
            return;
        }
    }
    if (flag) {
        element.SetValue(static_cast<T>(num));
        static_throw(_expression.Append(element));
    }
}

// L[͂𐔊wI\ɉ߂ȂB
// Ɏ̏ꍇB
template < >
inline Void Expression<Float64>::CreateMathExpression(KeySequenceConstRef keySeq) static_throws
{
    KeyType key;
    KeyType::KeyCode code;
    ExpressionElement<Float64> element;
    Float64 base(1.0), x(0.0);
    SInt32 i;
    SInt32Const size(keySeq.GetSize());
    Bool flagNum(false), flagPoint(false);
    SFXMTRandom r;
    UInt32 count(0);

    for (i = 0; i < size; ++i) {
        key = keySeq.GetKeyType(i);
        code = key.GetKeyCode();
        if (code & KeyType::KEYCLASS_NUMBER) {
            // lɕϊ
            if (!flagNum) {
                flagNum = true;
                flagPoint = false;
                x = static_cast<Float64>(code - KeyType::KEYCODE_NUMBER_0);
            }
            else {
                if (!flagPoint) {
                    x *= 10.0;
                    x += static_cast<Float64>(code - KeyType::KEYCODE_NUMBER_0);
                }
                else {
                    if (++count <= MAX_FLOAT_ORDER) {
                        x *= 10.0;
                        x += static_cast<Float64>(code - KeyType::KEYCODE_NUMBER_0);
                        base *= 10.0;
                    }
                }
            }
            continue;
        }
        else if (code == KeyType::KEYCODE_FLOAT_POINT) {
            if (flagPoint) {
                static_throw(Calculator::ERR_SYNTAX);
                return;
            }
            else {
                if (!flagNum) {
                    flagNum = true;
                    x = 0.0;
                }
                flagPoint = true;
                count = 0;
                base = 1.0;
            }
            continue;
        }
        if (flagNum) {
            if (flagPoint) {
                element.SetValue(x / base);
            }
            else {
                element.SetValue(x);
            }
            flagNum = false;
            flagPoint = false;
            count = 0;
            static_throw(_expression.Append(element));
            if (static_catch() != SFERR_NO_ERROR) {
                return;
            }
        }
        // 萔̓WJ
        if (code == KeyType::KEYCODE_CONSTANT_PI) {
            element.SetValue(MATH_CONSTANT_PI);
        }
        else if (code == KeyType::KEYCODE_CONSTANT_E) {
            element.SetValue(MATH_CONSTANT_E);
        }
        else if (code == KeyType::KEYCODE_CONSTANT_RAN) {
            element.SetValue(r.GetFloat64());
        }
        // Pʂ̓WJ
        else if (code & KeyType::KEYCLASS_UNIT) {
            _hasUnit = true;
            switch (code) {
                case KeyType::KEYCODE_UNIT_TERA:
                    element.SetValue(MATH_UNIT_TERA);
                    break;
                case KeyType::KEYCODE_UNIT_GIGA:
                    element.SetValue(MATH_UNIT_GIGA);
                    break;
                case KeyType::KEYCODE_UNIT_MEGA:
                    element.SetValue(MATH_UNIT_MEGA);
                    break;
                case KeyType::KEYCODE_UNIT_KILO:
                    element.SetValue(MATH_UNIT_KILO);
                    break;
                case KeyType::KEYCODE_UNIT_MILLI:
                    element.SetValue(MATH_UNIT_MILLI);
                    break;
                case KeyType::KEYCODE_UNIT_MICRO:
                    element.SetValue(MATH_UNIT_MICRO);
                    break;
                case KeyType::KEYCODE_UNIT_NANO:
                    element.SetValue(MATH_UNIT_NANO);
                    break;
                case KeyType::KEYCODE_UNIT_PICO:
                    element.SetValue(MATH_UNIT_PICO);
                    break;
                case KeyType::KEYCODE_UNIT_FEMTO:
                    element.SetValue(MATH_UNIT_FEMTO);
                    break;
            }
        }
        else if (code == KeyType::KEYCODE_FLOAT_EXP) {
            // w\
            element.SetType(ExpressionElement<Float64>::RAWTYPE_FEXP);
        }
        else if (code == KeyType::KEYCODE_MEMORY_ANS) {
            // 
            element.SetValue(_ans);
        }
        else {
            element.SetType(ExpressionElement<Float64>::InterpretKeyCode(code));
        }
        static_throw(_expression.Append(element));
        if (static_catch() != SFERR_NO_ERROR) {
            return;
        }
    }
    if (flagNum) {
        if (flagPoint) {
            element.SetValue(x / base);
        }
        else {
            element.SetValue(x);
        }
        static_throw(_expression.Append(element));
    }
}

// ʂ̐`FbN
template <typename T>
inline Void Expression<T>::CheckParenthesesCount(Void) const static_throws
{
    if (static_catch() != SFERR_NO_ERROR) {
        return;
    }

    SInt32 parentheses(0);
    UInt16 type;
    ExpressionElement<T> element;
    SInt32 i;
    SInt32Const size(_expression.GetSize());

    for (i = 0; i < size; ++i) {
        element = _expression.Get(i);
        type = element.GetType();
        if (type == ExpressionElement<T>::RAWTYPE_LEFT || ExpressionElement<T>::IsRawFunction(type)) {
            ++parentheses;
        }
        else if (type == ExpressionElement<T>::RAWTYPE_RIGHT) {
            --parentheses;
        }
    }
    if (parentheses != 0) {
        static_throw(Calculator::ERR_SYNTAX);
    }
}

// ֐Ȃǂ̐擪T
template <typename T>
inline SInt32 Expression<T>::GetStartPosition(SInt32 index) const
{
    SInt32 size(_expression.GetSize());

    if (index < 0) {
        index = 0;
    }
    if (index >= size) {
        index = size - 1;
    }

    ExpressionElement<T> element;
    UInt16 type;
    SInt32 parentheses(0);
    while (--index >= 0) {
        element = _expression.Get(index);
        type = element.GetType();
        if (parentheses == 0
            && (type == ExpressionElement<T>::RAWTYPE_LEFT || ExpressionElement<T>::IsRawFunction(type)
            || type == ExpressionElement<T>::RAWTYPE_SEPARATOR || ExpressionElement<T>::IsRawOperator(type))) {
            break;
        }
        if (type == ExpressionElement<T>::RAWTYPE_LEFT || ExpressionElement<T>::IsRawFunction(type)) {
            ++parentheses;
        }
        else if (type == ExpressionElement<T>::RAWTYPE_RIGHT) {
            --parentheses;
        }
    }
    return ++index;
}

// ֐Ȃǂ̏I[T (ł͂ȂA^̏I[)
template <typename T>
inline SInt32 Expression<T>::GetEndPosition(SInt32 index) const
{
    SInt32Const size(_expression.GetSize());

    if (index < 0) {
        index = 0;
    }
    if (index >= size) {
        index = size - 1;
    }

    ExpressionElement<T> element;
    UInt16 type;
    SInt32 parentheses(0);
    while (++index < size) {
        element = _expression.Get(index);
        type = element.GetType();
        if (parentheses == 0
            && (type == ExpressionElement<T>::RAWTYPE_RIGHT || type == ExpressionElement<T>::RAWTYPE_SEPARATOR || ExpressionElement<T>::IsRawOperator(type))) {
            break;
        }
        if (type == ExpressionElement<T>::RAWTYPE_LEFT || ExpressionElement<T>::IsRawFunction(type)) {
            ++parentheses;
        }
        else if (type == ExpressionElement<T>::RAWTYPE_RIGHT) {
            --parentheses;
        }
    }
    return --index;
}

// t|[h\LɕϊOɍsׂ
template <typename T>
inline Void Expression<T>::Preprocess(Void) static_throws
{
    SInt32 i;
    ExpressionElement<T> element, element2;

    // ṕuł *v
    for (i = 0; i < _expression.GetSize(); ++i) {
        element = _expression.Get(i);
        if (element.GetType() == ExpressionElement<T>::RAWTYPE_SMUL) {
            element2.SetType(ExpressionElement<T>::RAWTYPE_MUL);
            _expression.Set(i, element2);
            element2.SetType(ExpressionElement<T>::RAWTYPE_RIGHT);
            static_throw(_expression.Insert(GetEndPosition(i) + 1, element2));
            if (static_catch() == SFERR_NO_ERROR) {
                element2.SetType(ExpressionElement<T>::RAWTYPE_LEFT);
                static_throw(_expression.Insert(GetStartPosition(i++), element2));
            }
            if (static_catch() != SFERR_NO_ERROR) {
                return;
            }
        }
    }
}

// t|[h\LɕϊOɍsׂ
template < >
inline Void Expression<Float64>::Preprocess(Void) static_throws
{
    // 

    if (static_catch() != SFERR_NO_ERROR) {
        return;
    }

    SInt32 i, j;
    UInt16 type;
    ExpressionElement<Float64> element, element2;

    // w\ (-> * 10 ^ )
    for (i = _expression.GetSize() - 1; i >= 0; --i) {
        element = _expression.Get(i);
        if (element.GetType() == ExpressionElement<Float64>::RAWTYPE_FEXP) {
            element2.SetType(ExpressionElement<Float64>::RAWTYPE_LEFT);
            static_throw(_expression.Insert(GetStartPosition(i++), element2));
            if (static_catch() != SFERR_NO_ERROR) {
                return;
            }
            j = (i < _expression.GetSize() - 1) ? (i + 1) : i;
            element2 = _expression.Get(j);
            type = element2.GetType();
            if (type != ExpressionElement<Float64>::RAWTYPE_PLUS && type != ExpressionElement<Float64>::RAWTYPE_MINUS) {
                j = i;
            }
            element2.SetType(ExpressionElement<Float64>::RAWTYPE_RIGHT);
            static_throw(_expression.Insert(GetEndPosition(j) + 1, element2));
            if (static_catch() != SFERR_NO_ERROR) {
                return;
            }
            element2.SetType(ExpressionElement<Float64>::RAWTYPE_MUL);
            _expression.Set(i, element2);
            j = i + 1;
            element2.SetType(ExpressionElement<Float64>::RAWTYPE_OPOW);
            static_throw(_expression.Insert(j, element2));
            if (static_catch() == SFERR_NO_ERROR) {
                element2.SetValue(10.0);
                static_throw(_expression.Insert(j, element2));
            }
            if (static_catch() != SFERR_NO_ERROR) {
                return;
            }
        }
    }

    // ṕuł *v
    for (i = 0; i < _expression.GetSize(); ++i) {
        element = _expression.Get(i);
        if (element.GetType() == ExpressionElement<Float64>::RAWTYPE_SMUL) {
            element2.SetType(ExpressionElement<Float64>::RAWTYPE_MUL);
            _expression.Set(i, element2);
            element2.SetType(ExpressionElement<Float64>::RAWTYPE_RIGHT);
            static_throw(_expression.Insert(GetEndPosition(i) + 1, element2));
            if (static_catch() == SFERR_NO_ERROR) {
                element2.SetType(ExpressionElement<Float64>::RAWTYPE_LEFT);
                static_throw(_expression.Insert(GetStartPosition(i++), element2));
            }
            if (static_catch() != SFERR_NO_ERROR) {
                return;
            }
        }
    }
}

// PZqt|[h\Lɕϊ
template <typename T>
inline Void Expression<T>::ConvertUnaryOperator(Void) static_throws
{
    if (static_catch() != SFERR_NO_ERROR) {
        return;
    }

    ExpressionElement<T> element, element2, element3;
    UInt16 type, type2, type3;
    SInt32 i, j;

    // ! (K) ̏
    for (i = 0; i < _expression.GetSize(); ++i) {
        element = _expression.Get(i);
        type = element.GetType();
        if (type == ExpressionElement<T>::RAWTYPE_FACTORIAL) {
            element2.SetType(ExpressionElement<T>::TYPE_FACTORIAL);
            // !! (dK) ǂ̃`FbN
            j = i + 1;
            if (j < _expression.GetSize()) {
                element3 = _expression.Get(j);
                type3 = element3.GetType();
                if (type3 == ExpressionElement<T>::RAWTYPE_FACTORIAL) {
                    element2.SetType(ExpressionElement<T>::TYPE_DFACTORIAL);
                    _expression.Remove(j);
                }
            }
            _expression.Set(i, element2);
        }
    }

    // +A- () ̏
    for (i = _expression.GetSize() - 1; i >= 0; --i) {
        element = _expression.Get(i);
        type = element.GetType();
        if (type == ExpressionElement<T>::RAWTYPE_PLUS || type == ExpressionElement<T>::RAWTYPE_MINUS) {
            // +  -
            element2 = _expression.Get((i == 0) ? 0 : (i - 1));
            type2 = element2.GetType();
            if (type2 != ExpressionElement<T>::RAWTYPE_NONE && type2 != ExpressionElement<T>::RAWTYPE_VALUE
                && type2 != ExpressionElement<T>::RAWTYPE_RIGHT && !(type2 & ExpressionElement<T>::TYPE_PROCESSED)) {
                // ł
                element2.SetType((type == ExpressionElement<T>::RAWTYPE_PLUS) ? ExpressionElement<T>::TYPE_PLUS : ExpressionElement<T>::TYPE_MINUS);
                static_throw(_expression.Insert(GetEndPosition(i) + 1, element2));
                if (static_catch() != SFERR_NO_ERROR) {
                    return;
                }
                _expression.Remove(i);
            }
        }
    }

    // not ̏
    for (i = 0; i < _expression.GetSize(); ) {
        element = _expression.Get(i);
        type = element.GetType();
        if (type == ExpressionElement<T>::RAWTYPE_NOT) {
            element2.SetType(ExpressionElement<T>::TYPE_NOT);
            static_throw(_expression.Insert(GetEndPosition(i) + 1, element2));
            if (static_catch() != SFERR_NO_ERROR) {
                return;
            }
            _expression.Remove(i);
        }
        else {
            ++i;
        }
    }
}

// 2 Zqt|[h\Lɕϊ
template <typename T>
inline Void Expression<T>::ConvertBinaryOperator(Void) static_throws
{
    if (static_catch() != SFERR_NO_ERROR) {
        return;
    }

    ExpressionElement<T> element, element2;
    UInt16 type;
    SInt32 i;

    // ^ (ׂ) Zq
    for (i = _expression.GetSize() - 1; i >= 0; --i) {
        element = _expression.Get(i);
        type = element.GetType();
        if (type == ExpressionElement<T>::RAWTYPE_OPOW) {
            element2.SetType(ExpressionElement<T>::TYPE_POW);
            static_throw(_expression.Insert(GetEndPosition(i) + 1, element2));
            if (static_catch() != SFERR_NO_ERROR) {
                return;
            }
            _expression.Remove(i);
        }
    }

    // ̑ 2 Zq̏
    UInt16Const raw_type[] = {
        ExpressionElement<T>::RAWTYPE_NPR,
        ExpressionElement<T>::RAWTYPE_NCR,
        ExpressionElement<T>::RAWTYPE_DIV,
        ExpressionElement<T>::RAWTYPE_MUL,
        ExpressionElement<T>::RAWTYPE_MOD,
        ExpressionElement<T>::RAWTYPE_MINUS,
        ExpressionElement<T>::RAWTYPE_PLUS,
        ExpressionElement<T>::RAWTYPE_AND,
        ExpressionElement<T>::RAWTYPE_XOR,
        ExpressionElement<T>::RAWTYPE_OR
    };
    UInt16Const type_array[] = {
        ExpressionElement<T>::TYPE_NPR,
        ExpressionElement<T>::TYPE_NCR,
        ExpressionElement<T>::TYPE_DIV,
        ExpressionElement<T>::TYPE_MUL,
        ExpressionElement<T>::TYPE_MOD,
        ExpressionElement<T>::TYPE_SUB,
        ExpressionElement<T>::TYPE_ADD,
        ExpressionElement<T>::TYPE_AND,
        ExpressionElement<T>::TYPE_XOR,
        ExpressionElement<T>::TYPE_OR
    };
    UInt16 j;
    UInt16Const size(lengthof(type_array));
    for (j = 0; j < size; ++j) {
        for (i = 0; i < _expression.GetSize(); ) {
            element = _expression.Get(i);
            type = element.GetType();
            if (type == raw_type[j]) {
                element2.SetType(type_array[j]);
                static_throw(_expression.Insert(GetEndPosition(i) + 1, element2));
                if (static_catch() != SFERR_NO_ERROR) {
                    return;
                }
                _expression.Remove(i);
            }
            else {
                ++i;
            }
        }
    }
}

// ֐t|[h\Lɕϊ
// [hŎg邱Ƃ͂Ȃ
template <typename T>
inline Void Expression<T>::ConvertFunction(Void) static_throws
{
    // (A) 菜
    ExpressionElement<T> element;
    UInt16 type;
    SInt32 i;

    for (i = 0; i < _expression.GetSize(); ) {
        element = _expression.Get(i);
        type = element.GetType();
        if (type == ExpressionElement<T>::RAWTYPE_LEFT || type == ExpressionElement<T>::RAWTYPE_RIGHT) {
            _expression.Remove(i);
        }
        else {
            ++i;
        }
    }
}

// ֐t|[h\Lɕϊ
// [hp
template < >
inline Void Expression<Float64>::ConvertFunction(Void) static_throws
{
    if (static_catch() != SFERR_NO_ERROR) {
        return;
    }

    ExpressionElement<Float64> element, element2;
    UInt16 type;
    SInt32 i;

    // , 菜
    for (i = 0; i < _expression.GetSize(); ) {
        element = _expression.Get(i);
        type = element.GetType();
        if (type == ExpressionElement<Float64>::RAWTYPE_SEPARATOR) {
            _expression.Remove(i);
        }
        else {
            ++i;
        }
    }

    // ֐̓oʒu ) ̌ɂ
    for (i = 0; i < _expression.GetSize(); ) {
        element = _expression.Get(i);
        type = element.GetType();
        if (ExpressionElement<Float64>::IsRawFunction(type)) {
            element2.SetType(type - ExpressionElement<Float64>::RAWTYPE_FUNCTION + ExpressionElement<Float64>::TYPE_FUNCTION);
            static_throw(_expression.Insert(GetEndPosition(i) + 1, element2));
            if (static_catch() != SFERR_NO_ERROR) {
                return;
            }
            _expression.Remove(i);
        }
        else {
            ++i;
        }
    }

    // (A) 菜
    for (i = 0; i < _expression.GetSize(); ) {
        element = _expression.Get(i);
        type = element.GetType();
        if (type == ExpressionElement<Float64>::RAWTYPE_LEFT || type == ExpressionElement<Float64>::RAWTYPE_RIGHT) {
            _expression.Remove(i);
        }
        else {
            ++i;
        }
    }
}

template <typename T>
inline SInt32 Expression<T>::GetSize(Void) const
{
    return _expression.GetSize();
}

template <typename T>
inline const ExpressionElement<T> &Expression<T>::Get(SInt32 index) const
{
    return _expression.Get(index);
}

template <typename T>
inline Bool Expression<T>::HasUnit(Void) const
{
    return _hasUnit;
}

#endif // __SOPHIACRADLE_EXPRESSION_HPP //
