#include "SuperMath.hpp"

#ifdef EPSILON
#undef EPSILON
#endif
#ifdef PI
#undef PI
#endif
#ifdef LOG_2PI
#undef LOG_2PI
#endif
#ifdef N
#undef N
#endif
#ifdef B0
#undef B0
#endif
#ifdef B1
#undef B1
#endif
#ifdef B2
#undef B2
#endif
#ifdef B4
#undef B4
#endif
#ifdef B6
#undef B6
#endif
#ifdef B8
#undef B8
#endif
#ifdef B10
#undef B10
#endif
#ifdef B12
#undef B12
#endif
#ifdef B14
#undef B14
#endif
#ifdef B16
#undef B16
#endif

#define EPSILON     (0.001)
#define PI          (3.14159265358979324)
#define LOG_2PI     (1.83787706640934548)
#define N           (8)
#define B0          (1.0)
#define B1          (-1.0 / 2.0)
#define B2          (1.0 / 6.0)
#define B4          (-1.0 / 30.0)
#define B6          (1.0 / 42.0)
#define B8          (-1.0 / 30.0)
#define B10         (5.0 / 66.0)
#define B12         (-691.0 / 2730.0)
#define B14         (7.0 / 6.0)
#define B16         (-3617.0 / 510.0)

//#ifndef TARGET_COMPILER_ARMCPP
#if !defined(TARGET_COMPILER_ARMCPP) && !defined(TARGET_COMPILER_GNUCPP)
// Newton @ɂ 3 捪̌vZ
Float64 SuperMath::Cbrt(Float64 x)
{
    Float64 s, t, u;
    Bool minus(false);

    if (x == 0) {
        return 0;
    }
    if (x < 0) {
        minus = true;
        x = -x;
    }
    s = (x > 1) ? x : 1;
    do {
        u = s;
        t = s * s;
        s += (x - t * s) / (2.0 * t + x / s);
    } while (s < u);
    return minus ? -u : u;
}

// toȐ֐
Float64 SuperMath::Asinh(Float64 x)
{
    // ` : -inf < x < inf

    Float64 y;
    if (x > EPSILON) {
        y = log(sqrt(x * x + 1.0) + x);
    }
    else if (x < -EPSILON) {
        y = -log(sqrt(x * x + 1.0) - x);
    }
    else {
        y = x * (1.0 - x * x / 6.0);
    }
    return y;
}

Float64 SuperMath::Acosh(Float64 x)
{
    // ` : x >= 1
    return log(x + sqrt(x * x - 1.0));
}

Float64 SuperMath::Atanh(Float64 x)
{
    // ` : -1 < x < 1

    Float64 y;
    if (fabs(x) > EPSILON) {
        y = 0.5 * log((1.0 + x) / (1.0 - x));
    }
    else {
        y = x * (1.0 + x * x / 3.0);
    }
    return y;
}

// log Gamma(x)
Float64 SuperMath::Loggamma(Float64 x)
{
    Float64 v(1.0);
    while (x < N) {
        v *= (x++);
    }
    Float64Const w(1.0 / (x * x));
    return ((((((((B16 / (16.0 * 15.0)) * w + (B14 / (14.0 * 13.0))) * w
                + (B12 / (12.0 * 11.0))) * w + (B10 / (10.0 * 9.0))) * w
                + (B8 / (8.0 * 7.0))) * w + (B6 / (6.0 * 5.0))) * w
                + (B4 / (4.0 * 3.0))) * w + (B2 / (2.0 * 1.0))) / x
                + 0.5 * LOG_2PI - log(v) - x + (x - 0.5) * log(x);
}
#endif

// K}֐
Float64 SuperMath::Gamma(Float64 x)
{
    return 0.0;
    /*
    Float64 y;
    if (x < 0) {
        y = PI / (sin(PI * x) * exp(SuperMath::Loggamma(1.0 - x)));
    }
    else {
        y = exp(SuperMath::Loggamma(x));
    }
    return y;
    */
}

// x -> WA
Float64 SuperMath::FromDegToRad(Float64 x)
{
    return PI / 180.0 * x;
}

// OWA -> WA
Float64 SuperMath::FromGradToRad(Float64 x)
{
    return PI / 200.0 * x;
}

// WA -> x
Float64 SuperMath::FromRadToDeg(Float64 x)
{
    return 180.0 / PI * x;
}

// WA -> OWA
Float64 SuperMath::FromRadToGrad(Float64 x)
{
    return 200.0 / PI * x;
}

// 
Float64 SuperMath::Permutation(UInt32 n, UInt32 r)
{
    UInt32 i;
    Float64 x(1.0);
    for (i = 0; i < r; ++i) {
        x *= static_cast<Float64>(n - i);
        if (!Finite(x)) {
            break;
        }
    }
    return x;
}

// gݍ킹
Float64 SuperMath::Combination(UInt32 n, UInt32 r)
{
    UInt32 i;
    Float64 x(1.0);
    for (i = 0; i < r; ) {
        x *= static_cast<Float64>(n - i);
        x /= static_cast<Float64>(++i);
        if (!Finite(x)) {
            break;
        }
    }
    return x;
}

// dK
Float64 SuperMath::DoubleFactorial(UInt32 n)
{
    UInt32 i;
    Float64 x(1.0);
    for (i = ((n & 1U) ? 1 : 2); i <= n; i += 2) {
        x *= static_cast<Float64>(i);
        if (!Finite(x)) {
            break;
        }
    }
    return x;
}
