#ifndef __SOPHIACRADLE_SUPERMATH_HPP
#define __SOPHIACRADLE_SUPERMATH_HPP

#include <SophiaFramework.hpp>

// RpC̈ႢɂAw֐̒`̈Ⴂz邽߂̃NX
SFMTYPEDEFCLASS(SuperMath)
class SuperMath {
SFMSEALCONSTRUCT(SuperMath)
public:
    static Float64 Fabs(Float64 x);
    static Float64 Floor(Float64 x);
    static Float64 Fmod(Float64 x, Float64 y);
    static Float64 Pow(Float64 x, Float64 y);
    static Float64 Exp(Float64 x);
    static Float64 Log(Float64 x);
    static Float64 Log10(Float64 x);
    static Float64 Sqrt(Float64 x);
    static Float64 Cbrt(Float64 x);
    static Float64 Sin(Float64 x);
    static Float64 Cos(Float64 x);
    static Float64 Tan(Float64 x);
    static Float64 Asin(Float64 x);
    static Float64 Acos(Float64 x);
    static Float64 Atan(Float64 x);
    static Float64 Sinh(Float64 x);
    static Float64 Cosh(Float64 x);
    static Float64 Tanh(Float64 x);
    static Float64 Asinh(Float64 x);
    static Float64 Acosh(Float64 x);
    static Float64 Atanh(Float64 x);
    static Float64 Loggamma(Float64 x);
    static Float64 Gamma(Float64 x);
    static Float64 FromDegToRad(Float64 x);
    static Float64 FromGradToRad(Float64 x);
    static Float64 FromRadToDeg(Float64 x);
    static Float64 FromRadToGrad(Float64 x);
    static Float64 Permutation(UInt32 n, UInt32 r);
    static Float64 Combination(UInt32 n, UInt32 r);
    static Float64 DoubleFactorial(UInt32 n);
    static Float64 Jn(SInt32 n, Float64 x);
    static Float64 Yn(SInt32 n, Float64 x);
    static Bool Finite(Float64 x);
    static Bool Isnan(Float64 x);
};

inline Float64 SuperMath::Fabs(Float64 x)
{
    return fabs(x);
}

inline Float64 SuperMath::Floor(Float64 x)
{
    return floor(x);
}

inline Float64 SuperMath::Fmod(Float64 x, Float64 y)
{
    return fmod(x, y);
}

inline Float64 SuperMath::Pow(Float64 x, Float64 y)
{
    return pow(x, y);
}

inline Float64 SuperMath::Exp(Float64 x)
{
    return exp(x);
}

inline Float64 SuperMath::Log(Float64 x)
{
    return log(x);
}

inline Float64 SuperMath::Log10(Float64 x)
{
    return log10(x);
}

inline Float64 SuperMath::Sqrt(Float64 x)
{
    return sqrt(x);
}

#if defined(TARGET_ENVIRONMENT_PHYSICAL)
inline Float64 SuperMath::Cbrt(Float64 x)
{
    return cbrt(x);
}
#endif

inline Float64 SuperMath::Sin(Float64 x)
{
    return sin(x);
}

inline Float64 SuperMath::Cos(Float64 x)
{
    return cos(x);
}

inline Float64 SuperMath::Tan(Float64 x)
{
    return tan(x);
}

inline Float64 SuperMath::Asin(Float64 x)
{
    return asin(x);
}

inline Float64 SuperMath::Acos(Float64 x)
{
    return acos(x);
}

inline Float64 SuperMath::Atan(Float64 x)
{
    return atan(x);
}

inline Float64 SuperMath::Sinh(Float64 x)
{
    return sinh(x);
}

inline Float64 SuperMath::Cosh(Float64 x)
{
    return cosh(x);
}

inline Float64 SuperMath::Tanh(Float64 x)
{
    return tanh(x);
}

#if defined(TARGET_ENVIRONMENT_PHYSICAL)
inline Float64 SuperMath::Asinh(Float64 x)
{
    return asinh(x);
}

inline Float64 SuperMath::Acosh(Float64 x)
{
    return acosh(x);
}

inline Float64 SuperMath::Atanh(Float64 x)
{
    return atanh(x);
}

inline Float64 SuperMath::Loggamma(Float64 x)
{
    SIntN sign;
    return lgamma_r(x, &sign);
}
#endif

inline Float64 SuperMath::Jn(SInt32 n, Float64 x)
{
#if defined(TARGET_ENVIRONMENT_PHYSICAL)
    return jn(n, x);
#else
    return _jn(n, x);
#endif
}

inline Float64 SuperMath::Yn(SInt32 n, Float64 x)
{
#if defined(TARGET_ENVIRONMENT_PHYSICAL)
    return yn(n, x);
#else
    return _yn(n, x);
#endif
}

inline Bool SuperMath::Finite(Float64 x)
{
#if defined(TARGET_ENVIRONMENT_PHYSICAL)
    return finite(x) != 0;
#else
    return _finite(x) != 0;
#endif
}

inline Bool SuperMath::Isnan(Float64 x)
{
#if defined(TARGET_ENVIRONMENT_PHYSICAL)
    return isnan(x) != 0;
#else
    return _isnan(x) != 0;
#endif
}

#endif // __SOPHIACRADLE_SUPERMATH_HPP //
