# Modified by SFMAKMODIFIER Ver 1.0
#============================================================================
#  Name:
#    $(TARGET).MAK
#
#  Description:
#    Makefile to build the $(TARGET) downloadable module.
#
#   The following nmake targets are available in this makefile:
#
#     all           - make .elf and .mod image files (default)
#     clean         - delete object directory and image files
#     filename.o    - make object file
#
#   The above targets can be made with the following command:
#
#     nmake /f $(TARGET).mak [target]
#
#  Assumptions:
#    1. The environment variable ADSHOME is set to the root directory of the
#       arm tools.
#    2. The version of ADS is 1.2 or above.
#
#  Notes:
#    None.
#
#
#        Copyright  2000-2001 QUALCOMM Incorporated.
#               All Rights Reserved.
#            QUALCOMM Proprietary/GTDR
#
#----------------------------------------------------------------------------
#============================================================================
SF_HOME        =$(SFHOME)
ARM_HOME       =$(ARMHOME)
BREW_HOME      =$(BREWDIR)
TARGET         =psec
OBJS           =Calculator.o FNKeyboard.o FSKeyboard.o Keyboard.o KeySequence.o MNKeyboard.o Monitor.o ONKeyboard.o PDialog.o psec.o Softkey.o SuperMath.o TNKeyboard.o TSKeyboard.o VDialog.o VNKeyboard.o VSKeyboard.o VUKeyboard.o 
APP_INCLUDES   = -I "$(BREWDIR)\inc" -I "$(SF_HOME)\Include"

#-------------------------------------------------------------------------------
# Target file name and type definitions
#-------------------------------------------------------------------------------

EXETYPE    = elf                # Target image file format
MODULE     = mod                # Downloadable module extension

#-------------------------------------------------------------------------------
# Target compile time symbol definitions
#
# Tells the SDK source stuffs that we're building a dynamic app.
#-------------------------------------------------------------------------------

# DYNAPP          = -DDYNAMIC_APP
DYNAPP          = -DDYNAMIC_APP


#-------------------------------------------------------------------------------
# Software tool and environment definitions
#-------------------------------------------------------------------------------

AEESRCPATH = $(BREW_HOME)\src
AEEINCPATH = $(BREW_HOME)\inc

ARMBIN = $(ARM_HOME)\bin        # ARM ADS application directory
ARMINC = $(ARM_HOME)\include    # ARM ADS include file directory
ARMLIB = $(ARM_HOME)\lib        # ARM ADS library directory

ARMCC    = $(ARMBIN)\armcc      # ARM ADS ARM 32-bit inst. set ANSI CPP compiler
ARMCPP   = $(ARMBIN)\armcpp     # ARM ADS ARM 32-bit inst. set ANSI CPP compiler
LD       = $(ARMBIN)\armlink     # ARM ADS linker
HEXTOOL  = $(ARMBIN)\fromelf     # ARM ADS utility to create hex file from image

OBJ_CMD    = -o                 # Command line option to specify output filename

#-------------------------------------------------------------------------------
# Processor architecture options
#-------------------------------------------------------------------------------

CPU = -cpu ARM7TDMI             # ARM7TDMI target processor

#-------------------------------------------------------------------------------
# ARM Procedure Call Standard (APCS) options
#-------------------------------------------------------------------------------

ROPI     = ropi                 # Read-Only(code) Position independence
INTERWRK = interwork            # Allow ARM-Thumb interworking

APCS = -apcs /$(ROPI)/$(INTERWRK)/norwpi

#-------------------------------------------------------------------------------
# Additional compile time error checking options
#-------------------------------------------------------------------------------

CHK = -fa                       # Check for data flow anomolies

#-------------------------------------------------------------------------------
# Compiler output options
#-------------------------------------------------------------------------------

OUT = -c                        # Object file output only

#-------------------------------------------------------------------------------
# Compiler/assembler debug options
#-------------------------------------------------------------------------------

# DBG = -g                        # Enable debug
DBG = -g-

#-------------------------------------------------------------------------------
# Compiler optimization options
#-------------------------------------------------------------------------------

# OPT = -Ospace -O2               # Full compiler optimization for space
OPT = -Otime -O2

#-------------------------------------------------------------------------------
# Compiler code generation options
#-------------------------------------------------------------------------------

END = -littleend                # Compile for little endian memory architecture
ZA  = -zo                       # LDR may only access 32-bit aligned addresses

CODE = $(END) $(ZA)


#-------------------------------------------------------------------------------
# Include file search path options
#-------------------------------------------------------------------------------

INC = -I. -I$(AEEINCPATH) $(APP_INCLUDES)


#-------------------------------------------------------------------------------
# Compiler pragma emulation options
#-------------------------------------------------------------------------------


#-------------------------------------------------------------------------------
# Linker options
#-------------------------------------------------------------------------------

LINK_CMD = -o                    #Command line option to specify output file
                                 #on linking

ROPILINK = -ropi                 #Link image as Read-Only Position Independent

LINK_ORDER = -first AEEMod_Load

#-------------------------------------------------------------------------------
# HEXTOOL options
#-------------------------------------------------------------------------------

BINFORMAT = -bin


#-------------------------------------------------------------------------------
# Compiler flag definitions
#-------------------------------------------------------------------------------

CFLAGS0 = $(OUT) $(DYNAPP) $(CPU) $(APCS) $(CODE) $(CHK) $(DBG)
CFLAGS  = $(CFLAGS0) $(OPT)

#-------------------------------------------------------------------------------
# Linker flag definitions
#-------------------------------------------------------------------------------

# the -entry flag is not really needed, but it keeps the linker from reporting
# warning L6305W (no entry point).  The address
# LFLAGS = $(ROPILINK) -rwpi -entry 0x8000#
LFLAGS = $(ROPILINK) -rwpi -entry AEEMod_Load#0x8000#


#----------------------------------------------------------------------------
# Default target
#----------------------------------------------------------------------------

all :  $(TARGET).$(MODULE)

#----------------------------------------------------------------------------
# Clean target
#----------------------------------------------------------------------------

# The object subdirectory, target image file, and target hex file are deleted.

clean :
	@echo ---------------------------------------------------------------
	@echo CLEAN
	-del /f $(OBJS)
	-del /f $(TARGET).$(EXETYPE)
	-del /f $(TARGET).$(MODULE)
	@echo ---------------------------------------------------------------

#============================================================================
#                           DEFAULT SUFFIX RULES
#============================================================================

# The following are the default suffix rules used to compile all objects that
# are not specifically included in one of the module specific rules defined
# in the next section.

# The following macros are used to specify the output object file, MSG_FILE
# symbol definition and input source file on the compile line in the rules
# defined below.

SRC_CPP_FILE = $(@F:.o=.cpp)      # Input source file specification
SRC_C_FILE   = $(@F:.o=.c)      # Input source file specification
OBJ_FILE = $(OBJ_CMD) $(@F)   # Output object file specification

.SUFFIXES :
.SUFFIXES : .o .dep .c .cpp

#--------------------------------------------------------------------------
# C code inference rules
#----------------------------------------------------------------------------

.cpp.o:
	@echo ---------------------------------------------------------------
	@echo OBJECT $(@F)
	$(ARMCPP) $(CFLAGS) $(INC) $(OBJ_FILE) $(SRC_CPP_FILE)
	@echo ---------------------------------------------------------------


{$(AEESRCPATH)}.c.o:
	@echo ---------------------------------------------------------------
	@echo OBJECT $(@F)
	$(ARMCC) $(CFLAGS) $(INC) $(OBJ_FILE) $(AEESRCPATH)\$(SRC_C_FILE)
	@echo ---------------------------------------------------------------


#===============================================================================
#                           MODULE SPECIFIC RULES
#===============================================================================

APP_OBJS =  $(OBJS) "$(SF_HOME)\Library\SophiaFrameworkBrew400JpnRVCTB12.a"

#----------------------------------------------------------------------------
# Lib file targets
#----------------------------------------------------------------------------

$(TARGET).$(MODULE) : $(TARGET).$(EXETYPE)
	@echo ---------------------------------------------------------------
	@echo TARGET $@
	$(HEXTOOL)  $(TARGET).$(EXETYPE) $(BINFORMAT) $(TARGET).$(MODULE)

$(TARGET).$(EXETYPE) : $(APP_OBJS)
	@echo ---------------------------------------------------------------
	@echo TARGET $@
	$(LD) $(LINK_CMD) $(TARGET).$(EXETYPE) $(LFLAGS) $(APP_OBJS) $(LINK_ORDER)

#----------------------------------------------------------------------------
# Applet Specific Rules
#----------------------------------------------------------------------------


# --------------------------------------------
# DEPENDENCY LIST, DO NOT EDIT BELOW THIS LINE
# --------------------------------------------


calculator.o:	calculator.cpp
calculator.o:	calculator.hpp
calculator.o:	keysequence.hpp
calculator.o:	array.hpp
calculator.o:	cursor.hpp
calculator.o:	monitor.hpp
calculator.o:	calculator.hpp
calculator.o:	cursor.hpp
calculator.o:	expression.hpp
calculator.o:	array.hpp
calculator.o:	keysequence.hpp
calculator.o:	calculator.hpp
calculator.o:	sangi.hpp
calculator.o:	expression.hpp
calculator.o:	stack.hpp
calculator.o:	supermath.hpp
calculator.o:	supermath.hpp
fnkeyboard.o:	fnkeyboard.cpp
fnkeyboard.o:	fnkeyboard.hpp
fnkeyboard.o:	keyboard.hpp
fnkeyboard.o:	monitor.hpp
fnkeyboard.o:	calculator.hpp
fnkeyboard.o:	keysequence.hpp
fnkeyboard.o:	array.hpp
fnkeyboard.o:	cursor.hpp
fnkeyboard.o:	calculator.hpp
fnkeyboard.o:	fskeyboard.hpp
fnkeyboard.o:	keyboard.hpp
fnkeyboard.o:	tnkeyboard.hpp
fnkeyboard.o:	keyboard.hpp
fskeyboard.o:	fskeyboard.cpp
fskeyboard.o:	fskeyboard.hpp
fskeyboard.o:	keyboard.hpp
fskeyboard.o:	monitor.hpp
fskeyboard.o:	calculator.hpp
fskeyboard.o:	keysequence.hpp
fskeyboard.o:	array.hpp
fskeyboard.o:	cursor.hpp
fskeyboard.o:	calculator.hpp
fskeyboard.o:	fnkeyboard.hpp
fskeyboard.o:	keyboard.hpp
fskeyboard.o:	tskeyboard.hpp
fskeyboard.o:	keyboard.hpp
keyboard.o:	keyboard.cpp
keyboard.o:	keyboard.hpp
keyboard.o:	monitor.hpp
keyboard.o:	calculator.hpp
keyboard.o:	keysequence.hpp
keyboard.o:	array.hpp
keyboard.o:	cursor.hpp
keyboard.o:	calculator.hpp
keyboard.o:	psec_res.h
keyboard.o:	psec.hpp
keyboard.o:	psec.bid
keyboard.o:	calculator.hpp
keyboard.o:	softkey.hpp
keyboard.o:	mnkeyboard.hpp
keyboard.o:	keyboard.hpp
keyboard.o:	vnkeyboard.hpp
keyboard.o:	keyboard.hpp
keyboard.o:	pdialog.hpp
keyboard.o:	vdialog.hpp
keysequence.o:	keysequence.cpp
keysequence.o:	keysequence.hpp
keysequence.o:	array.hpp
keysequence.o:	cursor.hpp
keysequence.o:	calculator.hpp
keysequence.o:	keysequence.hpp
mnkeyboard.o:	mnkeyboard.cpp
mnkeyboard.o:	mnkeyboard.hpp
mnkeyboard.o:	keyboard.hpp
mnkeyboard.o:	monitor.hpp
mnkeyboard.o:	calculator.hpp
mnkeyboard.o:	keysequence.hpp
mnkeyboard.o:	array.hpp
mnkeyboard.o:	cursor.hpp
mnkeyboard.o:	calculator.hpp
mnkeyboard.o:	softkey.hpp
mnkeyboard.o:	vnkeyboard.hpp
mnkeyboard.o:	keyboard.hpp
mnkeyboard.o:	vskeyboard.hpp
mnkeyboard.o:	keyboard.hpp
mnkeyboard.o:	onkeyboard.hpp
mnkeyboard.o:	keyboard.hpp
monitor.o:	monitor.cpp
monitor.o:	monitor.hpp
monitor.o:	calculator.hpp
monitor.o:	keysequence.hpp
monitor.o:	array.hpp
monitor.o:	cursor.hpp
monitor.o:	psec_res.h
onkeyboard.o:	onkeyboard.cpp
onkeyboard.o:	onkeyboard.hpp
onkeyboard.o:	keyboard.hpp
onkeyboard.o:	monitor.hpp
onkeyboard.o:	calculator.hpp
onkeyboard.o:	keysequence.hpp
onkeyboard.o:	array.hpp
onkeyboard.o:	cursor.hpp
onkeyboard.o:	calculator.hpp
onkeyboard.o:	fnkeyboard.hpp
onkeyboard.o:	keyboard.hpp
pdialog.o:	pdialog.cpp
pdialog.o:	pdialog.hpp
pdialog.o:	psec.hpp
pdialog.o:	psec.bid
pdialog.o:	calculator.hpp
pdialog.o:	keysequence.hpp
pdialog.o:	array.hpp
pdialog.o:	cursor.hpp
pdialog.o:	softkey.hpp
psec.o:	psec.cpp
psec.o:	psec.hpp
psec.o:	psec.bid
psec.o:	calculator.hpp
psec.o:	keysequence.hpp
psec.o:	array.hpp
psec.o:	cursor.hpp
psec.o:	psec_res.h
psec.o:	softkey.hpp
psec.o:	monitor.hpp
psec.o:	calculator.hpp
psec.o:	mnkeyboard.hpp
psec.o:	keyboard.hpp
psec.o:	monitor.hpp
psec.o:	calculator.hpp
psec.o:	vnkeyboard.hpp
psec.o:	keyboard.hpp
psec.o:	vskeyboard.hpp
psec.o:	keyboard.hpp
psec.o:	vukeyboard.hpp
psec.o:	keyboard.hpp
psec.o:	onkeyboard.hpp
psec.o:	keyboard.hpp
psec.o:	fnkeyboard.hpp
psec.o:	keyboard.hpp
psec.o:	fskeyboard.hpp
psec.o:	keyboard.hpp
psec.o:	tnkeyboard.hpp
psec.o:	keyboard.hpp
psec.o:	tskeyboard.hpp
psec.o:	keyboard.hpp
softkey.o:	softkey.cpp
softkey.o:	softkey.hpp
supermath.o:	supermath.cpp
supermath.o:	supermath.hpp
tnkeyboard.o:	tnkeyboard.cpp
tnkeyboard.o:	tnkeyboard.hpp
tnkeyboard.o:	keyboard.hpp
tnkeyboard.o:	monitor.hpp
tnkeyboard.o:	calculator.hpp
tnkeyboard.o:	keysequence.hpp
tnkeyboard.o:	array.hpp
tnkeyboard.o:	cursor.hpp
tnkeyboard.o:	calculator.hpp
tnkeyboard.o:	tskeyboard.hpp
tnkeyboard.o:	keyboard.hpp
tnkeyboard.o:	fnkeyboard.hpp
tnkeyboard.o:	keyboard.hpp
tskeyboard.o:	tskeyboard.cpp
tskeyboard.o:	tskeyboard.hpp
tskeyboard.o:	keyboard.hpp
tskeyboard.o:	monitor.hpp
tskeyboard.o:	calculator.hpp
tskeyboard.o:	keysequence.hpp
tskeyboard.o:	array.hpp
tskeyboard.o:	cursor.hpp
tskeyboard.o:	calculator.hpp
tskeyboard.o:	tnkeyboard.hpp
tskeyboard.o:	keyboard.hpp
tskeyboard.o:	fskeyboard.hpp
tskeyboard.o:	keyboard.hpp
vdialog.o:	vdialog.cpp
vdialog.o:	vdialog.hpp
vdialog.o:	psec_res.h
vdialog.o:	psec.hpp
vdialog.o:	psec.bid
vdialog.o:	calculator.hpp
vdialog.o:	keysequence.hpp
vdialog.o:	array.hpp
vdialog.o:	cursor.hpp
vdialog.o:	softkey.hpp
vnkeyboard.o:	vnkeyboard.cpp
vnkeyboard.o:	vnkeyboard.hpp
vnkeyboard.o:	keyboard.hpp
vnkeyboard.o:	monitor.hpp
vnkeyboard.o:	calculator.hpp
vnkeyboard.o:	keysequence.hpp
vnkeyboard.o:	array.hpp
vnkeyboard.o:	cursor.hpp
vnkeyboard.o:	calculator.hpp
vnkeyboard.o:	vskeyboard.hpp
vnkeyboard.o:	keyboard.hpp
vnkeyboard.o:	onkeyboard.hpp
vnkeyboard.o:	keyboard.hpp
vskeyboard.o:	vskeyboard.cpp
vskeyboard.o:	vskeyboard.hpp
vskeyboard.o:	keyboard.hpp
vskeyboard.o:	monitor.hpp
vskeyboard.o:	calculator.hpp
vskeyboard.o:	keysequence.hpp
vskeyboard.o:	array.hpp
vskeyboard.o:	cursor.hpp
vskeyboard.o:	calculator.hpp
vskeyboard.o:	vukeyboard.hpp
vskeyboard.o:	keyboard.hpp
vskeyboard.o:	onkeyboard.hpp
vskeyboard.o:	keyboard.hpp
vukeyboard.o:	vukeyboard.cpp
vukeyboard.o:	vukeyboard.hpp
vukeyboard.o:	keyboard.hpp
vukeyboard.o:	monitor.hpp
vukeyboard.o:	calculator.hpp
vukeyboard.o:	keysequence.hpp
vukeyboard.o:	array.hpp
vukeyboard.o:	cursor.hpp
vukeyboard.o:	calculator.hpp
vukeyboard.o:	onkeyboard.hpp
vukeyboard.o:	keyboard.hpp
